/*************************************/
/* i2c.c                             */
/* I2c bus for MSP430f149            */
/* (in Software!!!)                  */
/*************************************/
/* Functions defined in this file:   */
/*    i2cInit ()                     */
/*    i2cStart()                     */
/*    i2cStop ()                     */
/*    i2cRead ()                     */
/*    i2cWrite()                     */
/* Functions internal to this file   */
/*    SetLowSDA ()                   */
/*    SetLowSCL ()                   */
/*    SetHighSDA()                   */
/*    SetHighSCL()                   */
/*************************************/

#include <msp430x43x.h>
#include "i2c.h"

#define SDA       0x01  // Serial Data Line
#define SCL       0x02  // Serial Clock Line

/******************************************************************************/
/*  SDA == P2.0                                                               */
/*  SCL == P2.1                                                               */
/******************************************************************************/

/*************************/

static void SetLowSDA()
{
   P2DIR |= SDA;           
}
static void SetHighSDA()
{
   P2DIR &= ~SDA;         
}

static void SetLowSCL()
{
   P2OUT &= ~SCL;
}
static void SetHighSCL()
{
   P2OUT |= SCL;
}
/*************************/
/* To initialize the bus */
/*************************/
void i2cInit()
{
   P2SEL &=  ~SDA;
   P2SEL &=  ~SCL; 

   P2OUT &= ~SCL;   
   P2OUT &= ~SDA;   

   P2DIR |= SCL;    
   P2DIR &= ~SDA;    

   SetHighSCL();
   SetLowSDA(); 
   SetHighSDA();
}

/*************************/
/* Start Data Transfer   */
/*************************/
void i2cStart()
{

  SetHighSCL();
  SetHighSDA();

  SetHighSCL();
  SetLowSDA();

  SetLowSCL();
  SetHighSDA();
}
/*************************/
/* Stop  Transfer        */
/*************************/
void i2cStop()
{
  SetLowSCL();
  SetLowSDA();

  SetHighSCL();
  SetLowSDA();

  SetHighSCL();
  SetHighSDA();
}
/*************************/
/* Write  Transfer       */
/*************************/
int i2cWrite(char a)
{
  int i;
  int return_ack;

  for (i=0; i<8; i++){
     SetLowSCL();
     if (a & 0x80)          
       SetHighSDA();
     else
       SetLowSDA();

     SetHighSCL();
     a <<= 1;
  }
  SetLowSCL();

  /* ack Read */
  SetHighSDA();

  SetHighSCL();

  if (P2IN & SDA)         
   return_ack = NO_I2C_ACK;      
  else
   return_ack = OK_I2C_ACK;
  SetLowSCL();
  return(return_ack);
}
/*************************/
/* Read  Transfer        */
/*************************/
char  i2cRead(int ack)
{
  int i;
  char caracter = 0x00;

  SetLowSCL();
  SetHighSDA();

  for (i=0; i<8; i++){
     caracter = caracter << 1;
     SetHighSCL();
     if (P2IN & SDA)
       caracter = caracter  + 1;
     SetLowSCL();
  }

  if (ack){                          
     SetLowSDA();
  }
  SetHighSCL();
  SetLowSCL();

  return (caracter);
}
